import 'dart:ffi' as ffi;

import 'bindings.dart' as ffi;
import 'dylib.dart';
import 'int8str.dart';
import 'variant_type.dart';

class GSettingsSchema {
  final ffi.Pointer<ffi.GSettingsSchema> _ptr;

  GSettingsSchema.fromId(String id) : _ptr = _lookupSchema(id);

  GSettingsSchema.fromPointer(this._ptr);
  ffi.Pointer<ffi.GSettingsSchema> toPointer() => _ptr;

  static ffi.Pointer<ffi.GSettingsSchema> _lookupSchema(String id) {
    final cid = id.newInt8String();
    final source = dylib.g_settings_schema_source_get_default();
    final schema = dylib.g_settings_schema_source_lookup(source, cid, 1);
    cid.dispose();
    return schema;
  }

  String get id => dylib.g_settings_schema_get_id(_ptr).toDartString();
  String get path => dylib.g_settings_schema_get_path(_ptr).toDartString();

  bool hasKey(String name) {
    final cname = name.newInt8String();
    final res = dylib.g_settings_schema_has_key(_ptr, cname) != 0;
    cname.dispose();
    return res;
  }

  GSettingsSchemaKey key(String name) {
    final cname = name.newInt8String();
    final ckey = dylib.g_settings_schema_get_key(_ptr, cname);
    cname.dispose();
    return GSettingsSchemaKey.fromPointer(ckey);
  }
}

class GSettingsSchemaKey {
  final ffi.Pointer<ffi.GSettingsSchemaKey> _ptr;

  GSettingsSchemaKey.fromPointer(this._ptr) {
    dylib.g_settings_schema_key_ref(_ptr);
  }

  ffi.Pointer<ffi.GSettingsSchemaKey> toPointer() => _ptr;

  void dispose() => dylib.g_settings_schema_key_unref(_ptr);

  String get name => dylib.g_settings_schema_key_get_name(_ptr).toDartString();

  String get summary {
    return dylib.g_settings_schema_key_get_summary(_ptr).toDartString();
  }

  String get description {
    return dylib.g_settings_schema_key_get_description(_ptr).toDartString();
  }

  String get typeName {
    return dylib.g_settings_schema_key_get_value_type(_ptr).typeName;
  }
}
